//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Scheduling;
using MindFusion.Scheduling.WinForms;
using MindFusion.Text;


namespace MindFusion.Scheduling.WinForms.Samples.CS.InfoHeader
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			MindFusion.Drawing.GdiGraphics g =
				new MindFusion.Drawing.GdiGraphics(CreateGraphics());

			aText1.Setup("This is a <b>formatted</b> string.", g, aFont);
			aText2.Setup("This is <color=#009900>another</color> formatted string.", g, aFont);
			aText3.Setup("<i>Info message</i>.", g, aFont);
			aText4.Setup("This is <color=#ff0000><u>important</u></color>.", g, aFont);

			g.Dispose();
		}

		private void Calendar_Draw(object sender, DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.TimetableInfoHeader)
			{
				Rectangle b = e.Bounds;

				b.Width -= 1;
				b.Height -= 1;
				b.Inflate(-1, -1);
				e.Graphics.DrawRectangle(Pens.Yellow, b);

				RectangleF r = new RectangleF(b.X, b.Y, b.Width, b.Height);
				LayoutOptions options = new LayoutOptions();

				options.Alignment = StringAlignment.Near;
				options.LineAlignment = StringAlignment.Near;

				// Select the appropriate text according to the info
				// header currently being drawn
				MindFusion.Text.Text theText = null;
				if (e.Date.Date == new DateTime(2007, 5, 8))
				{
					if (e.Resource.Name == "contact1")
						theText = aText1;
					if (e.Resource.Name == "contact2")
						theText = aText2;
				}
				else if (e.Date.Date == new DateTime(2007, 5, 9))
				{
					if (e.Resource.Name == "contact1")
						theText = aText3;
					if (e.Resource.Name == "contact2")
						theText = aText4;
				}

				if (theText == null)
					return;

				aLayout.LayoutInRectangle(theText, r, options, e.Graphics, aFont);
				aLayout.Draw(0, 0, new RenderTextCallback(this.RenderText),
					new DrawTextHint(e.Graphics, aFont, Brushes.Black, StringFormat.GenericDefault, true));
			}
		}

		private void RenderText(string text, RectangleF dest, DrawTextHint hint)
		{
			hint.Graphics.DrawString(text, hint.Font, hint.Brush, dest.X, dest.Y);
		}

		private void Calendar_TooltipDisplaying(object sender, TooltipEventArgs e)
		{
			if (e.Element == CalendarElement.TimetableInfoHeader)
			{
				// Select the appropriate tooltip
				MindFusion.Text.Text theText = null;
				if (e.Time.Date == new DateTime(2007, 5, 8))
				{
					if (e.Resource.Name == "contact1")
						theText = aText1;
					if (e.Resource.Name == "contact2")
						theText = aText2;
				}
				else if (e.Time.Date == new DateTime(2007, 5, 9))
				{
					if (e.Resource.Name == "contact1")
						theText = aText3;
					if (e.Resource.Name == "contact2")
						theText = aText4;
				}
				e.Tooltip = theText.PlainText;
			}
		}

		private void Button1_Click(object sender, EventArgs e)
		{
			if (calendar.TimetableSettings.ShowInfoHeader == State.Enabled)
				calendar.TimetableSettings.ShowInfoHeader = State.Disabled;
			else
				calendar.TimetableSettings.ShowInfoHeader = State.Enabled;
		}


		private string _appPath;
		private StyledText aText1 = new StyledText();
		private StyledText aText2 = new StyledText();
		private StyledText aText3 = new StyledText();
		private StyledText aText4 = new StyledText();
		private Layout aLayout = new Layout();
		private Font aFont = new Font("Tahoma", 8);
	}
}
